# SPDX-License-Identifier: GPL-2.0-only
# Copyright (c) 2019-2022 NITK Surathkal

"""
The base exception for all exceptions created in NeST
"""


class NestBaseException(Exception):
    """
    Base exception for all exceptions created in NeST
    """
